@@ -16,63 +16,59 @@
 """
 Parse various kinds of 'CVS notify' email.
 """
+
 import calendar
 import datetime
 import re
 import time
-
 from email import message_from_file
 from email.iterators import body_line_iterator
 from email.utils import mktime_tz
 from email.utils import parseaddr
 from email.utils import parsedate_tz
 
+from twisted.internet import defer
+from twisted.python import log
+from zope.interface import implementer
+
 from buildbot import util
 from buildbot.interfaces import IChangeSource
 from buildbot.util.maildir import MaildirService
-from twisted.internet import defer
-from twisted.python import log
-from zope.interface import implements
 
 
+@implementer(IChangeSource)
 class MaildirSource(MaildirService, util.ComparableMixin):
 
     """Generic base class for Maildir-based change sources"""
-    implements(IChangeSource)
 
-    compare_attrs = ["basedir", "pollinterval", "prefix"]
+    compare_attrs = ("basedir", "pollinterval", "prefix")
+    name = 'MaildirSource'
 
     def __init__(self, maildir, prefix=None, category='', repository=''):
-        MaildirService.__init__(self, maildir)
+        super().__init__(maildir)
         self.prefix = prefix
         self.category = category
         self.repository = repository
         if prefix and not prefix.endswith("/"):
-            log.msg("%s: you probably want your prefix=('%s') to end with "
+            log.msg(f"MaildirSource: you probably want your prefix=('{prefix}') to end with "
                     "a slash")
 
     def describe(self):
-        return "%s watching maildir '%s'" % (self.__class__.__name__, self.basedir)
+        return f"{self.__class__.__name__} watching maildir '{self.basedir}'"
 
+    @defer.inlineCallbacks
     def messageReceived(self, filename):
-        d = defer.succeed(None)
+        with self.moveToCurDir(filename) as f:
+            chtuple = self.parse_file(f, self.prefix)
 
-        def parse_file(_):
-            f = self.moveToCurDir(filename)
-            return self.parse_file(f, self.prefix)
-        d.addCallback(parse_file)
-
-        def add_change(chtuple):
-            src, chdict = None, None
-            if chtuple:
-                src, chdict = chtuple
-            if chdict:
-                return self.master.addChange(src=src, **chdict)
-            else:
-                log.msg("no change found in maildir file '%s'" % filename)
-        d.addCallback(add_change)
-
-        return d
+        src = None
+        chdict = None
+        if chtuple:
+            src, chdict = chtuple
+        if chdict:
+            yield self.master.data.updates.addChange(src=str(src), **chdict)
+        else:
+            log.msg(f"no change found in maildir file '{filename}'")
 
     def parse_file(self, fd, prefix=None):
         m = message_from_file(fd)
@@ -83,8 +79,10 @@ class CVSMaildirSource(MaildirSource):
     name = "CVSMaildirSource"
 
     def __init__(self, maildir, prefix=None, category='',
-                 repository='', properties={}):
-        MaildirSource.__init__(self, maildir, prefix, category, repository)
+                 repository='', properties=None):
+        super().__init__(maildir, prefix, category, repository)
+        if properties is None:
+            properties = {}
         self.properties = properties
 
     def parse(self, m, prefix=None):
@@ -94,14 +92,16 @@ class CVSMaildirSource(MaildirSource):
         # local username is enough to identify them (this assumes a one-server
         # cvs-over-rsh environment rather than the server-dirs-shared-over-NFS
         # model)
-        name, addr = parseaddr(m["from"])
+        _, addr = parseaddr(m["from"])
         if not addr:
-            return None  # no From means this message isn't from buildbot-cvs-mail
+            # no From means this message isn't from buildbot-cvs-mail
+            return None
         at = addr.find("@")
         if at == -1:
             author = addr  # might still be useful
         else:
             author = addr[:at]
+        author = util.bytes2unicode(author, encoding="ascii")
 
         # CVS accepts RFC822 dates. buildbot-cvs-mail adds the date as
         # part of the mail header, so use that.
@@ -117,7 +117,7 @@ class CVSMaildirSource(MaildirSource):
         else:
             when = mktime_tz(dateTuple)
 
-        theTime = datetime.datetime.utcfromtimestamp(float(when))
+        theTime = datetime.datetime.fromtimestamp(float(when), datetime.timezone.utc)
         rev = theTime.strftime('%Y-%m-%d %H:%M:%S')
 
         catRE = re.compile(r'^Category:\s*(\S.*)')
@@ -161,7 +161,7 @@ class CVSMaildirSource(MaildirSource):
             m = modRE.match(line)
             if m:
                 # We don't actually use this
-                #module = m.group(1)
+                # module = m.group(1)
                 continue
             m = pathRE.match(line)
             if m:
@@ -178,7 +178,7 @@ class CVSMaildirSource(MaildirSource):
             m = updateRE.match(line)
             if m:
                 # We don't actually use this
-                #updateof = m.group(1)
+                # updateof = m.group(1)
                 continue
             if line == "Log Message:\n":
                 break
@@ -214,19 +214,23 @@ class CVSMaildirSource(MaildirSource):
             if m:
                 path = m.group(1)
             else:
-                log.msg('CVSMaildirSource can\'t get path from file list. Ignoring mail')
-                return
+                log.msg(
+                    'CVSMaildirSource can\'t get path from file list. Ignoring mail')
+                return None
             fileList = fileList[len(path):].strip()
-            singleFileRE = re.compile(r'(.+?),(NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+)),(NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+))(?: |$)')
+            singleFileRE = re.compile(
+                r'(.+?),(NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+)),(NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+))(?: |$)')  # noqa pylint: disable=line-too-long
         elif cvsmode == '1.12':
-            singleFileRE = re.compile(r'(.+?) (NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+)) (NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+))(?: |$)')
+            singleFileRE = re.compile(
+                r'(.+?) (NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+)) (NONE|(?:\d+\.(?:\d+\.\d+\.)*\d+))(?: |$)')  # noqa pylint: disable=line-too-long
             if path is None:
-                raise ValueError('CVSMaildirSource cvs 1.12 require path. Check cvs loginfo config')
+                raise ValueError(
+                    'CVSMaildirSource cvs 1.12 require path. Check cvs loginfo config')
         else:
-            raise ValueError('Expected cvsmode 1.11 or 1.12. got: %s' % cvsmode)
+            raise ValueError(f'Expected cvsmode 1.11 or 1.12. got: {cvsmode}')
 
-        log.msg("CVSMaildirSource processing filelist: %s" % fileList)
-        while(fileList):
+        log.msg(f"CVSMaildirSource processing filelist: {fileList}")
+        while fileList:
             m = singleFileRE.match(fileList)
             if m:
                 curFile = path + '/' + m.group(1)
@@ -243,11 +247,20 @@ class CVSMaildirSource(MaildirSource):
         comments = comments.rstrip() + "\n"
         if comments == '\n':
             comments = None
-        return ('cvs', dict(author=author, files=files, comments=comments,
-                            isdir=isdir, when=when, branch=branch,
-                            revision=rev, category=category,
-                            repository=cvsroot, project=project,
-                            properties=self.properties))
+        return ('cvs', {
+            "author": author,
+            "committer": None,
+            "files": files,
+            "comments": comments,
+            "isdir": isdir,
+            "when": when,
+            "branch": branch,
+            "revision": rev,
+            "category": category,
+            "repository": cvsroot,
+            "project": project,
+            "properties": self.properties
+        })
 
 # svn "commit-email.pl" handler.  The format is very similar to freshcvs mail;
 # here's a sample:
@@ -286,7 +299,7 @@ class SVNCommitEmailMaildirSource(MaildirSource):
         # local username is enough to identify them (this assumes a one-server
         # cvs-over-rsh environment rather than the server-dirs-shared-over-NFS
         # model)
-        name, addr = parseaddr(m["from"])
+        _, addr = parseaddr(m["from"])
         if not addr:
             return None  # no From means this message isn't from svn
         at = addr.find("@")
@@ -301,7 +314,7 @@ class SVNCommitEmailMaildirSource(MaildirSource):
         # better to work with, unless you count pulling the v1-vs-v2
         # timestamp out of the diffs, which would be ugly. TODO: Pulling the
         # 'Date:' header from the mail is a possibility, and
-        # email.Utils.parsedate_tz may be useful. It should be configurable,
+        # email.utils.parsedate_tz may be useful. It should be configurable,
         # however, because there are a lot of broken clocks out there.
         when = util.now()
 
@@ -329,15 +342,13 @@ class SVNCommitEmailMaildirSource(MaildirSource):
             # source server's expected TZ setting! messy.
 
             # this stanza ends with the "Log:"
-            if (line == "Log:\n"):
+            if line == "Log:\n":
                 break
 
         # commit message is terminated by the file-listing section
         while lines:
             line = lines.pop(0)
-            if (line == "Modified:\n" or
-                line == "Added:\n" or
-                    line == "Removed:\n"):
+            if line in ("Modified:\n", "Added:\n", "Removed:\n"):
                 break
             comments += line
         comments = comments.rstrip() + "\n"
@@ -362,8 +373,8 @@ class SVNCommitEmailMaildirSource(MaildirSource):
                     if f.startswith(prefix):
                         f = f[len(prefix):]
                     else:
-                        log.msg("ignored file from svn commit: prefix '%s' "
-                                "does not match filename '%s'" % (prefix, f))
+                        log.msg(f"ignored file from svn commit: prefix '{prefix}' "
+                                f"does not match filename '{f}'")
                         continue
 
                 # TODO: figure out how new directories are described, set
@@ -374,8 +385,14 @@ class SVNCommitEmailMaildirSource(MaildirSource):
             log.msg("no matching files found, ignoring commit")
             return None
 
-        return ('svn', dict(author=author, files=files, comments=comments,
-                            when=when, revision=rev))
+        return ('svn', {
+            "author": author,
+            "committer": None,
+            "files": files,
+            "comments": comments,
+            "when": when,
+            "revision": rev
+        })
 
 # bzr Launchpad branch subscription mails. Sample mail:
 #
@@ -400,7 +417,8 @@ class SVNCommitEmailMaildirSource(MaildirSource):
 #   https://code.launchpad.net/~knielsen/maria/tmp-buildbot-test
 #
 #   You are subscribed to branch lp:~knielsen/maria/tmp-buildbot-test.
-#   To unsubscribe from this branch go to https://code.launchpad.net/~knielsen/maria/tmp-buildbot-test/+edit-subscription.
+#   To unsubscribe from this branch go to
+#   https://code.launchpad.net/~knielsen/maria/tmp-buildbot-test/+edit-subscription.
 #
 # [end of mail]
 
@@ -408,12 +426,12 @@ class SVNCommitEmailMaildirSource(MaildirSource):
 class BzrLaunchpadEmailMaildirSource(MaildirSource):
     name = "Launchpad"
 
-    compare_attrs = ["branchMap", "defaultBranch"]
+    compare_attrs = ("branchMap", "defaultBranch")
 
     def __init__(self, maildir, prefix=None, branchMap=None, defaultBranch=None, **kwargs):
         self.branchMap = branchMap
         self.defaultBranch = defaultBranch
-        MaildirSource.__init__(self, maildir, prefix, **kwargs)
+        super().__init__(maildir, prefix, **kwargs)
 
     def parse(self, m, prefix=None):
         """Parse branch notification messages sent by Launchpad.
@@ -428,7 +446,7 @@ class BzrLaunchpadEmailMaildirSource(MaildirSource):
 
         # Put these into a dictionary, otherwise we cannot assign them
         # from nested function definitions.
-        d = {'files': [], 'comments': u""}
+        d = {'files': [], 'comments': ""}
         gobbler = None
         rev = None
         author = None
@@ -438,25 +456,25 @@ class BzrLaunchpadEmailMaildirSource(MaildirSource):
             d['comments'] += s + "\n"
 
         def gobble_removed(s):
-            d['files'].append('%s REMOVED' % s)
+            d['files'].append(f'{s} REMOVED')
 
         def gobble_added(s):
-            d['files'].append('%s ADDED' % s)
+            d['files'].append(f'{s} ADDED')
 
         def gobble_modified(s):
-            d['files'].append('%s MODIFIED' % s)
+            d['files'].append(f'{s} MODIFIED')
 
         def gobble_renamed(s):
             match = re.search(r"^(.+) => (.+)$", s)
             if match:
-                d['files'].append('%s RENAMED %s' % (match.group(1), match.group(2)))
+                d['files'].append(f'{match.group(1)} RENAMED {match.group(2)}')
             else:
-                d['files'].append('%s RENAMED' % s)
+                d['files'].append(f'{s} RENAMED')
 
         lines = list(body_line_iterator(m, True))
         rev = None
         while lines:
-            line = unicode(lines.pop(0), "utf-8", errors="ignore")
+            line = str(lines.pop(0), "utf-8", errors="ignore")
 
             # revno: 101
             match = re.search(r"^revno: ([0-9.]+)", line)
@@ -471,13 +489,15 @@ class BzrLaunchpadEmailMaildirSource(MaildirSource):
             # timestamp: Fri 2009-05-15 10:35:43 +0200
             # datetime.strptime() is supposed to support %z for time zone, but
             # it does not seem to work. So handle the time zone manually.
-            match = re.search(r"^timestamp: [a-zA-Z]{3} (\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}) ([-+])(\d{2})(\d{2})$", line)
+            match = re.search(
+                r"^timestamp: [a-zA-Z]{3} (\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}) ([-+])(\d{2})(\d{2})$", line)  # noqa pylint: disable=line-too-long
             if match:
                 datestr = match.group(1)
                 tz_sign = match.group(2)
                 tz_hours = match.group(3)
                 tz_minutes = match.group(4)
-                when = parseLaunchpadDate(datestr, tz_sign, tz_hours, tz_minutes)
+                when = parseLaunchpadDate(
+                    datestr, tz_sign, tz_hours, tz_minutes)
 
             if re.search(r"^message:\s*$", line):
                 gobbler = gobble_comment
@@ -497,7 +517,7 @@ class BzrLaunchpadEmailMaildirSource(MaildirSource):
         if self.branchMap and repository:
             if repository in self.branchMap:
                 branch = self.branchMap[repository]
-            elif ("lp:" + repository) in self.branchMap:
+            elif "lp:" + repository in self.branchMap:
                 branch = self.branchMap['lp:' + repository]
         if not branch:
             if self.defaultBranch:
@@ -509,12 +529,17 @@ class BzrLaunchpadEmailMaildirSource(MaildirSource):
                     branch = None
 
         if rev and author:
-            return ('bzr', dict(author=author, files=d['files'],
-                                comments=d['comments'],
-                                when=when, revision=rev,
-                                branch=branch, repository=repository or ''))
-        else:
-            return None
+            return ('bzr', {
+                "author": author,
+                "committer": None,
+                "files": d['files'],
+                "comments": d['comments'],
+                "when": when,
+                "revision": rev,
+                "branch": branch,
+                "repository": repository or ''
+            })
+        return None
 
 
 def parseLaunchpadDate(datestr, tz_sign, tz_hours, tz_minutes):
