@@ -14,7 +14,8 @@
 # Copyright Buildbot Team Members
 
 import re
-from pkg_resources import parse_version
+
+from packaging.version import parse as parse_version
 
 from twisted.internet import defer
 from twisted.python import log
@@ -108,8 +109,7 @@ class GitMixin:
             env['GIT_SSH_COMMAND'] = ssh_command
         else:
             if sshWrapperPath is None:
-                raise Exception('Only SSH wrapper script is supported but path '
-                                'not given')
+                raise RuntimeError('Only SSH wrapper script is supported but path not given')
             env['GIT_SSH'] = sshWrapperPath
 
 
